@@ -59,6 +59,7 @@ slurm_persist_conn_t *slurmdbd_conn = NULL;
 
 static pthread_mutex_t agent_lock = PTHREAD_MUTEX_INITIALIZER;
 static pthread_cond_t  agent_cond = PTHREAD_COND_INITIALIZER;
+static pthread_cond_t shutdown_cond = PTHREAD_COND_INITIALIZER;
 static List      agent_list     = (List) NULL;
 static pthread_t agent_tid      = 0;
 
@@ -133,7 +134,8 @@ static int _unpack_return_code(uint16_t rpc_version, buf_t *buffer)
 		slurm_persist_free_rc_msg(msg);
 		break;
 	default:
-		error("bad message type %d != PERSIST_RC", msg_type);
+		error("bad message type %s != PERSIST_RC",
+		      slurmdbd_msg_type_2_str(msg_type, true));
 	}
 
 	return rc;
@@ -148,7 +150,7 @@ static int _get_return_code(void)
 
 	rc = _unpack_return_code(slurmdbd_conn->version, buffer);
 
-	free_buf(buffer);
+	FREE_NULL_BUFFER(buffer);
 	return rc;
 }
 
@@ -188,7 +190,7 @@ static int _handle_mult_rc_ret(void)
 					break;
 
 				if ((b = list_dequeue(agent_list))) {
-					free_buf(b);
+					FREE_NULL_BUFFER(b);
 				} else {
 					error("DBD_GOT_MULT_MSG "
 					      "unpack message error");
@@ -232,11 +234,12 @@ static int _handle_mult_rc_ret(void)
 			error("unpack message error");
 		break;
 	default:
-		error("bad message type %d != PERSIST_RC", msg_type);
+		error("bad message type %s != PERSIST_RC",
+		      slurmdbd_msg_type_2_str(msg_type, true));
 	}
 
 unpack_error:
-	free_buf(buffer);
+	FREE_NULL_BUFFER(buffer);
 	return rc;
 }
 
@@ -276,7 +279,7 @@ static buf_t *_load_dbd_rec(int fd)
 			continue;
 		else {
 			error("state recover error: %m");
-			free_buf(buffer);
+			FREE_NULL_BUFFER(buffer);
 			return NULL;
 		}
 	}
@@ -285,7 +288,7 @@ static buf_t *_load_dbd_rec(int fd)
 	rd_size = read(fd, &magic, size);
 	if ((rd_size != size) || (magic != DBD_MAGIC)) {
 		error("state recover error");
-		free_buf(buffer);
+		FREE_NULL_BUFFER(buffer);
 		return NULL;
 	}
 
@@ -322,7 +325,7 @@ static void _load_dbd_state(void)
 		safe_unpackstr_xmalloc(&ver_str, &ver_str_len, buffer);
 		debug3("Version string in dbd_state header is %s", ver_str);
 	unpack_error:
-		free_buf(buffer);
+		FREE_NULL_BUFFER(buffer);
 		buffer = NULL;
 		if (ver_str) {
 			/* get the version after VER */
@@ -349,7 +352,7 @@ static void _load_dbd_state(void)
 				set_buf_offset(buffer, 0);
 				rc = unpack_slurmdbd_msg(
 					&msg, rpc_version, buffer);
-				free_buf(buffer);
+				FREE_NULL_BUFFER(buffer);
 				if (rc == SLURM_SUCCESS)
 					buffer = pack_slurmdbd_msg(
 						&msg, SLURM_PROTOCOL_VERSION);
@@ -360,8 +363,7 @@ static void _load_dbd_state(void)
 				error("no buffer given");
 				continue;
 			}
-			if (!list_enqueue(agent_list, buffer))
-				fatal("list_enqueue, no memory");
+			list_enqueue(agent_list, buffer);
 			recovered++;
 			buffer = NULL;
 		}
@@ -431,7 +433,7 @@ static void _save_dbd_state(void)
 		buffer = init_buf(strlen(curr_ver_str));
 		packstr(curr_ver_str, buffer);
 		rc = _save_dbd_rec(fd, buffer);
-		free_buf(buffer);
+		FREE_NULL_BUFFER(buffer);
 		if (rc != SLURM_SUCCESS)
 			goto end_it;
 
@@ -444,19 +446,19 @@ static void _save_dbd_state(void)
 			 */
 			offset = get_buf_offset(buffer);
 			if (offset < 2) {
-				free_buf(buffer);
+				FREE_NULL_BUFFER(buffer);
 				continue;
 			}
 			set_buf_offset(buffer, 0);
 			(void) unpack16(&msg_type, buffer);  /* checked by offset */
 			set_buf_offset(buffer, offset);
 			if (msg_type == DBD_REGISTER_CTLD) {
-				free_buf(buffer);
+				FREE_NULL_BUFFER(buffer);
 				continue;
 			}
 
 			rc = _save_dbd_rec(fd, buffer);
-			free_buf(buffer);
+			FREE_NULL_BUFFER(buffer);
 			if (rc != SLURM_SUCCESS)
 				break;
 			wrote++;
@@ -635,10 +637,14 @@ static void *_agent(void *x)
 			slurm_mutex_unlock(&slurmdbd_lock);
 			_max_dbd_msg_action(&cnt);
 			END_TIMER2("slurmdbd agent: sleep");
-			log_flag(AGENT, "slurmdbd agent sleeping with agent_count=%d",
-				 list_count(agent_list));
 			abs_time.tv_sec  = time(NULL) + 10;
 			abs_time.tv_nsec = 0;
+			if (*slurmdbd_conn->shutdown != 0) {
+				slurm_mutex_unlock(&agent_lock);
+				break;
+			}
+			log_flag(AGENT, "slurmdbd agent sleeping with agent_count=%d",
+				 list_count(agent_list));
 			slurm_cond_timedwait(&agent_cond, &agent_lock,
 					     &abs_time);
 			slurm_mutex_unlock(&agent_lock);
@@ -730,7 +736,7 @@ static void *_agent(void *x)
 			} else
 				buffer = list_dequeue(agent_list);
 
-			free_buf(buffer);
+			FREE_NULL_BUFFER(buffer);
 			fail_time = 0;
 		} else {
 			/* We need to free a mult_msg even on failure */
@@ -738,7 +744,7 @@ static void *_agent(void *x)
 				if (list_msg.my_list != agent_list)
 					FREE_NULL_LIST(list_msg.my_list);
 				list_msg.my_list = NULL;
-				free_buf(buffer);
+				FREE_NULL_BUFFER(buffer);
 			}
 
 			fail_time = time(NULL);
@@ -761,6 +767,7 @@ static void *_agent(void *x)
 
 	FREE_NULL_LIST(agent_list);
 	agent_running = false;
+	slurm_cond_signal(&shutdown_cond);
 	slurm_mutex_unlock(&agent_lock);
 	return NULL;
 }
@@ -789,29 +796,10 @@ static void _shutdown_agent(void)
 		return;
 
 	slurmdbd_shutdown = time(NULL);
-	for (int i = 0; i < 50; i++) {	/* up to 5 secs total */
-		slurm_mutex_lock(&agent_lock);
-		if (!agent_running) {
-			slurm_mutex_unlock(&agent_lock);
-			goto fini;
-		}
+	slurm_mutex_lock(&agent_lock);
+	if (agent_running)
 		slurm_cond_broadcast(&agent_cond);
-		slurm_mutex_unlock(&agent_lock);
-
-		usleep(100000);	/* 0.1 sec per try */
-	}
-
-	/*
-	 * On rare occasions agent thread may not end quickly,
-	 * perhaps due to communication problems with slurmdbd.
-	 * Cancel it and join before returning or we could remove
-	 * and leave the agent without valid data.
-	 */
-	error("agent failed to shutdown gracefully");
-	error("unable to save pending requests");
-	pthread_cancel(agent_tid);
-
-fini:
+	slurm_mutex_unlock(&agent_lock);
 	pthread_join(agent_tid,  NULL);
 	agent_tid = 0;
 }
@@ -864,12 +852,6 @@ extern int slurmdbd_agent_send_recv(uint16_t rpc_version,
 
 	xassert(req);
 	xassert(resp);
-	xassert(slurmdbd_conn);
-
-	if (req->conn && (req->conn != slurmdbd_conn))
-		error("We are overriding the connection!!!!!");
-
-	req->conn = slurmdbd_conn;
 
 	/*
 	 * To make sure we can get this to send instead of the agent
@@ -878,8 +860,20 @@ extern int slurmdbd_agent_send_recv(uint16_t rpc_version,
 	 */
 	halt_agent = 1;
 	slurm_mutex_lock(&slurmdbd_lock);
+
 	halt_agent = 0;
 
+	if (!slurmdbd_conn) {
+		slurm_cond_signal(&slurmdbd_cond);
+		slurm_mutex_unlock(&slurmdbd_lock);
+		return ESLURM_DB_CONNECTION_INVALID;
+	}
+
+	if (req->conn && (req->conn != slurmdbd_conn))
+		error("We are overriding the connection!!!!!");
+
+	req->conn = slurmdbd_conn;
+
 	rc = dbd_conn_send_recv_direct(rpc_version, req, resp);
 
 	slurm_cond_signal(&slurmdbd_cond);
@@ -915,7 +909,7 @@ extern int slurmdbd_agent_send(uint16_t rpc_version, persist_msg_t *req)
 		_create_agent();
 		if ((agent_tid == 0) || (agent_list == NULL)) {
 			slurm_mutex_unlock(&agent_lock);
-			free_buf(buffer);
+			FREE_NULL_BUFFER(buffer);
 			return SLURM_ERROR;
 		}
 	}
@@ -934,15 +928,14 @@ extern int slurmdbd_agent_send(uint16_t rpc_version, persist_msg_t *req)
 	_max_dbd_msg_action(&cnt);
 
 	if (cnt < slurm_conf.max_dbd_msgs) {
-		if (list_enqueue(agent_list, buffer) == NULL)
-			fatal("list_enqueue: memory allocation failure");
+		list_enqueue(agent_list, buffer);
 	} else {
 		error("agent queue is full (%u), discarding %s:%u request",
 		      cnt,
 		      slurmdbd_msg_type_2_str(req->msg_type, 1),
 		      req->msg_type);
 		(slurmdbd_conn->trigger_callbacks.acct_full)();
-		free_buf(buffer);
+		FREE_NULL_BUFFER(buffer);
 		rc = SLURM_ERROR;
 	}
 
