@@ -56,6 +56,9 @@ extern int update_resv(resv_desc_msg_t *resv_desc_ptr, char **err_msg);
 /* Delete an existing resource reservation */
 extern int delete_resv(reservation_name_msg_t *resv_desc_ptr);
 
+extern void reservation_delete_resv_exc_parts(resv_exc_t *resv_exc);
+extern void reservation_delete_resv_exc(resv_exc_t *resv_exc);
+
 /* Return pointer to the named reservation or NULL if not found */
 extern slurmctld_resv_t *find_resv_name(char *resv_name);
 
@@ -169,7 +172,8 @@ extern uint32_t job_test_watts_resv(job_record_t *job_ptr, time_t when,
  * IN move_time    - if true, then permit the start time to advance from
  *                   "when" as needed IF job has no reservervation
  * OUT node_bitmap - nodes which the job can use, caller must free
- * OUT exc_core_bitmap - cores which the job can NOT use, caller must free
+ * OUT resv_exc_ptr - Various TRES (cores) which the job can NOT use, caller
+ *                    must free (reservation_delete_resv_exc[parts])
  * OUT resv_overlap - set to true if the job's run time and available nodes
  *		      overlap with an advanced reservation, indicates that
  *		      resources were removed from availability to the job
@@ -183,7 +187,7 @@ extern uint32_t job_test_watts_resv(job_record_t *job_ptr, time_t when,
  */
 extern int job_test_resv(job_record_t *job_ptr, time_t *when,
 			 bool move_time, bitstr_t **node_bitmap,
-			 bitstr_t **exc_core_bitmap, bool *resv_overlap,
+			 resv_exc_t *resv_exc_ptr, bool *resv_overlap,
 			 bool reboot);
 
 /*
